@@ -1,7 +1,7 @@
 #include "mainwindow.h"
 #include "ui_mainwindow.h"
-#include "loadprev.h"
 #include "memorycardmanager.h"
+#include "savestateselect.h"
 
 QString game;
 #ifdef Q_WS_MAEMO_5
@@ -11,6 +11,9 @@ QString cfg_path= QString("%1/.config/Team16/psx4m/").arg(QDir::homePath());
 #endif
 QString bios_path="/home/user/MyDocs/";
 QString rom_path="/home/user/MyDocs";
+QString sstate_path="/home/user/.pcsx/sstates/";
+QString screenshots_path="/opt/maemo/usr/games/screenshots/";
+
 MainWindow::MainWindow(QWidget *parent) :
     QMainWindow(parent),
     ui(new Ui::MainWindow)
@@ -49,7 +52,7 @@ MainWindow::MainWindow(QWidget *parent) :
 
     connect (ui->pushButton_2,SIGNAL(clicked()),this,SLOT(rom_config()));
     connect (ui->pushButton,SIGNAL(clicked()),this,SLOT(rom_exec()));
-
+    connect (ui->pbnSstate,SIGNAL(clicked()),this,SLOT(rom_exec_savestate()));
 
     this->menuBar()->addAction("Default config",this,SLOT(def_cfg()));
     this->menuBar()->addAction("GUI config",this,SLOT(gui_cfg()));
@@ -63,18 +66,18 @@ MainWindow::MainWindow(QWidget *parent) :
 void MainWindow::get_roms()
 {
     qDebug() << "Reading roms from" << rom_path;
-    roms.clear();
+    m_Roms.clear();
     ui->listWidget->clear();
     QDir dir(rom_path);
     dir.setFilter(QDir::Files);
     dir.setSorting(QDir::Name | QDir::IgnoreCase);
-    dir.setNameFilters(QStringList()<<"*.bin"<<"*.iso"<<"*.img"<<"*.z"<<"*.znx");
+    dir.setNameFilters(QStringList()<<"*.bin"<<"*.iso"<<"*.img"<<"*.z"<<"*.znx"<<"*.pbp");
 
     QFileInfoList list = dir.entryInfoList();
-    int o=0;
-    for (int i = 0; i < list.size(); ++i) {
-        QFileInfo fileInfo = list.at(i);
+    foreach (QFileInfo fileInfo, list){
         if (fileInfo.fileName()=="scph1001.bin") continue;
+
+        RomInfo* rom = new RomInfo();
         qDebug() << "  Rom:" << fileInfo.fileName();
 
         QIcon icon(":/images/ICON0.PNG");
@@ -83,44 +86,51 @@ void MainWindow::get_roms()
         else if (QFile::exists(QString("%1png").arg(fileInfo.filePath().replace(fileInfo.suffix(), ""))))
             icon = QIcon(QString("%1png").arg(fileInfo.filePath().replace(fileInfo.suffix(), "")));
         ui->listWidget->addItem(new QListWidgetItem(icon, fileInfo.fileName()));
-        roms.insert(o++,fileInfo.absoluteFilePath());
+
+        rom->FilePath = fileInfo.absoluteFilePath();
+        rom->FileName = fileInfo.fileName();
+        m_Roms.append(rom);
     }
-    if (roms.count() > 0){
+    if (m_Roms.count() > 0){
         ui->listWidget->setCurrentRow(0);
         currentRowChanged(0);
     }
 }
 
-void MainWindow::rom_exec(){
-    if (ui->listWidget->currentRow()!=-1){
-        if (current_core=="pcsx"){
-            QDir sc("/opt/maemo/usr/games/screenshots");
-            QStringList st=sc.entryList(QStringList(ui->listWidget->currentItem()->text()+".*"),QDir::NoFilter,QDir::NoSort);
-            if (st.count()>1){
-                bool ok;
-                QList <int> savest;
-                foreach(QString s,st)
-                savest.append(s.split(".").last().toInt(&ok, 10));
+void MainWindow::rom_exec_savestate()
+{
+    SavestateSelect* dlg = new SavestateSelect(this, m_Roms.at(ui->listWidget->currentRow()));
+    if (dlg->exec() == QDialog::Accepted){
+        int slot = dlg->Selected();
+        rom_exec(slot);
+    }
 
-                loadPrev *pr=new loadPrev(this);
-                pr->init(current_config,current_core,roms.value(ui->listWidget->currentRow()),savest);
-                pr->show();
-                return;
-            }
-        }
+}
+
+void MainWindow::rom_exec(){
+    rom_exec(-1);
+}
+
+void MainWindow::rom_exec(int slot)
+{
+    if (ui->listWidget->currentRow()!=-1){
+        game=ui->listWidget->currentItem()->text();
+        cfg->load_cfg();
 
         if (myProcess == NULL){
             ui->pushButton->setEnabled(false);
             ui->pushButton_2->setEnabled(false);
+            ui->pbnSstate->setEnabled(false);
             myProcess = new QProcess(this);
             myProcess->setWorkingDirectory("/home/user/");
             connect(myProcess, SIGNAL(finished(int)), this, SLOT(process_finished(int)));
 
-            QStringList args=current_config;
-            //args<<roms.value(ui->listWidget->currentRow())<<"-save_name"<<(ui->listWidget->currentItem()->text());
-            args<<roms.value(ui->listWidget->currentRow());
-            qDebug (QString("%1 %2").arg("/opt/maemo/usr/games/"+current_core).arg(args.join(" ")).toAscii());
+            QStringList args = current_config;
+            args << m_Roms.at(ui->listWidget->currentRow())->FilePath;
+            if (slot >= 0)
+                args << "-load" << QString::number(slot);
 
+            qDebug (QString("%1 %2").arg("/opt/maemo/usr/games/"+current_core).arg(args.join(" ")).toAscii());
             myProcess->start("/opt/maemo/usr/games/"+current_core, args);
             if (!myProcess->waitForStarted(10000)){
                 qDebug() << "Failed to start process.";
@@ -134,6 +144,7 @@ void MainWindow::process_finished(int)
 {
     ui->pushButton->setEnabled(true);
     ui->pushButton_2->setEnabled(true);
+    ui->pbnSstate->setEnabled(true);
     myProcess->deleteLater();
     myProcess = NULL;
 }
@@ -145,6 +156,37 @@ void MainWindow::currentRowChanged ( int  ){
         game=ui->listWidget->currentItem()->text();
         cfg->load_cfg();
 
+        RomInfo* rom = m_Roms.at(ui->listWidget->currentRow());
+        if (rom->SaveStateName.isEmpty()){
+            if (current_core == "pcsx"){
+                QProcess* p = new QProcess(this);
+                p->setWorkingDirectory("/home/user/");
+                QStringList args = QStringList() << "-cdfile" << rom->FilePath << "-get_sstatename" << "0";
+                p->start("/opt/maemo/usr/games/pcsx", args);
+                if (p->waitForFinished(3000)){
+                    QString out = p->readAll();
+                    QStringList lines = out.split("\n", QString::SkipEmptyParts);
+                    foreach (QString line, lines){
+                        line = line.trimmed();
+                        if (line.startsWith("SAVESTATE:")){
+                            QFileInfo f(line.remove(0, 8).trimmed());
+                            rom->SaveStateName = f.fileName();
+                            break;
+                        }
+                    }
+                }else{
+                    p->kill();
+                }
+                delete p;
+            }else{
+                QFileInfo f(rom->FileName);
+                rom->SaveStateName = QString("%1.save.0").arg(f.fileName());
+            }
+        }
+
+        int count = count_save_states(m_Roms.at(ui->listWidget->currentRow())->SaveStateName);
+        ui->pbnSstate->setEnabled(count > 0);
+
 /*        QString scr = QString("%1/%2_bkg.png").arg(rom_path).arg(game);
         if (QFile::exists(scr)){
             ui->listWidget->setStyleSheet(QString("QListView { \
@@ -272,10 +314,16 @@ MainWindow::~MainWindow()
 void MainWindow::rom_config()
 {
     if (ui->listWidget->currentRow()!=-1){
+        QString core = current_core;
         game=ui->listWidget->currentItem()->text();
         cfg->load_cfg();
         cfg->setWindowTitle(tr("Config: %1").arg(game));
         cfg->show();
+        if (core != current_core){
+            RomInfo* rom = m_Roms.at(ui->listWidget->currentRow());
+            rom->SaveStateName.clear();
+            currentRowChanged(ui->listWidget->currentRow());
+        }
     }
 }
 
@@ -285,3 +333,22 @@ void MainWindow::memory_cards()
     dlg->show();
     dlg = NULL;
 }
+
+int MainWindow::count_save_states(QString name)
+{
+    int res = 0;
+
+    QFileInfo f(name);
+    QDir dir;
+    if (current_core == "pcsx")
+        dir.setPath(sstate_path);
+    else
+        dir.setPath(rom_path);
+    dir.setFilter(QDir::Files);
+    dir.setSorting(QDir::Name | QDir::IgnoreCase);
+    dir.setNameFilters(QStringList()<< QString("%1*").arg(f.completeBaseName()));
+    QFileInfoList list = dir.entryInfoList();
+
+    res = list.count();
+    return res;
+}
