@@ -1,7 +1,7 @@
 #ifndef MAINWINDOW_H
 #define MAINWINDOW_H
 
-
+#include "rominfo.h"
 #include "options.h"
 #include "keys.h"
 
@@ -22,6 +22,7 @@ public:
 public slots:
     void rom_config();
     void rom_exec();
+    void rom_exec_savestate();
 
     void def_cfg();
     void gui_cfg();
@@ -35,10 +36,12 @@ public slots:
     void memory_cards();
 
 private:
+    void rom_exec(int slot);
     void get_roms();
+    int count_save_states(QString name);
 
     Ui::MainWindow *ui;
-    QStringList roms;
+    QList<RomInfo*> m_Roms;
     options *cfg;
     keys *keyopt;
     QLabel* lbl1;
