@@ -27,14 +27,21 @@ options::options(QWidget *parent) :
         ui->comboBox_4->addItem(fileInfo.fileName());
     }
     load_cfg();
+
+    m_Loop = NULL;
 }
 
 options::~options()
 {
     delete ui;
+
+    if (m_Loop)
+        delete m_Loop;
 }
 void options::hideEvent(QHideEvent *){
     if (!this->isVisible()){
+        m_Loop->quit();
+
         QDialog *dlg=new QDialog(this);
         dlg->setWindowTitle(tr("Confirm"));
         QGridLayout *lay=new QGridLayout(dlg);
@@ -395,7 +402,7 @@ void options::psx4m_hwa_block(){
 void options::memory_card_1()
 {
 #ifdef Q_WS_MAEMO_5
-    QString filename = GetMemoryCardName();
+    QString filename = GetMemoryCardName(m_MemoryCard1);
 #else
     QString filename = QFileDialog::getOpenFileName(this, tr("Open Memory card"),
                                                     QDir::homePath(),
@@ -411,7 +418,7 @@ void options::memory_card_1()
 void options::memory_card_2()
 {
 #ifdef Q_WS_MAEMO_5
-    QString filename = GetMemoryCardName();
+    QString filename = GetMemoryCardName(m_MemoryCard2);
 #else
     QString filename = QFileDialog::getOpenFileName(this, tr("Open Memory card"),
                                                     QDir::homePath(),
@@ -425,9 +432,9 @@ void options::memory_card_2()
 }
 
 #ifdef Q_WS_MAEMO_5
-QString options::GetMemoryCardName()
+QString options::GetMemoryCardName(QString selected)
 {
-    MemoryCardSelect* dlg = new MemoryCardSelect(this);
+    MemoryCardSelect* dlg = new MemoryCardSelect(this, selected);
     if (dlg->exec() == QDialog::Accepted)
         return dlg->Selected();
     return "";
@@ -444,4 +451,8 @@ void options::show()
     ui->scrollArea_4->verticalScrollBar()->setValue(0);
 
     QWidget::show();
+
+    m_Loop = new QEventLoop(this);
+    m_Loop->exec();
+
 }
