@@ -34,8 +34,9 @@ private:
     QString m_MemoryCard1;
     QString m_MemoryCard2;
 #ifdef Q_WS_MAEMO_5
-    QString GetMemoryCardName();
+    QString GetMemoryCardName(QString selected);
 #endif
+    QEventLoop* m_Loop;
 };
 
 #endif // OPTIONS_H
